/*
 * File:   main.c
 * Author: piotrek
 *
 * Created on April 8, 2016, 11:32 PM
 */
#include <string.h>
#include <stdio.h>
#include <xc.h>

#include <stdint.h>
#include <pic16f685.h>

#include "global.h"
#include "init.h"
#include "interrupts.h"
#include "led.h"
#include "switch.h"
#include "irdecoder.h"
#include "test_frame.h"
#include "menu.h"
    
eeprom ee_record_t ee_record;
main_state_machine_t main_sm;

static void set_analog_mux();
static void handle_mux();
static void handle_t1_ovf();
static void handle_ir_decoder();

void main(void) {

    
	hw_init ();
	sw_init ();	

	while(1) {
        CLRWDT();
        handle_ir_decoder();
        handle_t1_ovf();
        handle_menu();
        handle_disp();
	}
    
    
}

static void set_analog_mux() {
    static const uint8_t mux_map[] = {
        0xFF,
        ~(1 << 6),
        ~(1 << 6) & ~(1 << 5),
        ~(1 << 5)
    };
    PORTB |= (1 << 6) | (1 << 5);
    PORTB &= mux_map[main_sm.current_channel];

}
    
static void handle_t1_ovf() {
    if(main_sm.t1_ovf) {
        handle_switch();
        if(main_sm.frame_cnt) main_sm.frame_cnt--;
        dec_counter_exec(main_sm.timer_post_event, 1, set_event(POST_EVENT));
        dec_counter_exec(main_sm.sleep_timer, main_sm.sleep_timer_step, set_event(SLEEP_EVENT));
        dec_counter_exec(main_sm.mute_timer, 1, set_analog_mux());
        function_disp();
        main_sm.t1_ovf = 0;
    }    
}

static void handle_ir_decoder() {
    if (main_sm.ir_pulse_received) {
        sm_facade(main_sm.tpulse, !(PORTAbits.RA2));
        main_sm.ir_pulse_received = 0;
    }

    if(frame_ret.pt) {
        if(!main_sm.frame_cnt) {
            set_event(IR_RCVD);
        }
        main_sm.frame_cnt = FRAME_CNT_LVL;
    }
}
